@@ -123,9 +123,7 @@ class _Group(CommandObject):
         """
         for index, obj in enumerate(self.layouts):
             if obj.name == layout:
-                self.current_layout = index
-                hook.fire("layout_change", self.layouts[self.current_layout], self)
-                self.layout_all()
+                self.use_layout(index)
                 return
         logger.error("No such layout: %s", layout)
 
@@ -135,8 +133,9 @@ class _Group(CommandObject):
         self.current_layout = index % len(self.layouts)
         hook.fire("layout_change", self.layouts[self.current_layout], self)
         self.layout_all()
-        screen_rect = self.screen.get_rect()
-        self.layout.show(screen_rect)
+        if self.screen is not None:
+            screen_rect = self.screen.get_rect()
+            self.layout.show(screen_rect)
 
     def use_next_layout(self):
         self.use_layout((self.current_layout + 1) % (len(self.layouts)))
@@ -271,6 +270,7 @@ class _Group(CommandObject):
 
     def remove(self, win, force=False):
         self.windows.remove(win)
+        hook.fire("group_window_remove", self, win)
         hadfocus = self._remove_from_focus_history(win)
         win.group = None
 
@@ -362,7 +362,7 @@ class _Group(CommandObject):
             for i in self.windows:
                 if i.wid == sel:
                     return i
-        raise RuntimeError("Invalid selection: {}".format(name))
+        raise RuntimeError(f"Invalid selection: {name}")
 
     @expose_command()
     def setlayout(self, layout):
@@ -572,4 +572,4 @@ class _Group(CommandObject):
         hook.fire("changegroup")
 
     def __repr__(self):
-        return "<group.Group (%r)>" % self.name
+        return f"<group.Group ({self.name!r})>"
