@@ -29,37 +29,176 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
+from __future__ import annotations
 
 import asyncio
 import contextlib
+from typing import TYPE_CHECKING
 
-from libqtile import utils
+from libqtile import backend, utils
 from libqtile.log_utils import logger
 from libqtile.resources.sleep import inhibitor
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
+    HookHandler = Callable[[Callable], Callable]
+
 subscriptions = {}  # type: dict
-SKIPLOG = set()  # type: set
 
 
 def clear():
     subscriptions.clear()
 
 
-class Subscribe:
-    def __init__(self):
-        hooks = set([])
-        for i in dir(self):
-            if not i.startswith("_"):
-                hooks.add(i)
-        self.hooks = hooks
+def _fire_async_event(co):
+    from libqtile.utils import create_task
 
-    def _subscribe(self, event, func):
-        lst = subscriptions.setdefault(event, [])
+    loop = None
+    with contextlib.suppress(RuntimeError):
+        loop = asyncio.get_running_loop()
+
+    if loop is None:
+        asyncio.run(co)
+    else:
+        create_task(co)
+
+
+# Custom hook functions receive a single argument, "self", which will refer to the
+# Subscribe/Unsubscribe classes.
+
+
+def _resume_func(self):
+    def f(func):
+        inhibitor.want_resume()
+        return self._subscribe("resume", func)
+
+    return f
+
+
+def _suspend_func(self):
+    def f(func):
+        inhibitor.want_sleep()
+        return self._subscribe("suspend", func)
+
+    return f
+
+
+def _user_hook_func(self):
+    def wrapper(hook_name):
+        def f(func):
+            name = f"user_{hook_name}"
+            if name not in self.hooks:
+                self.hooks[name] = None
+            return self._subscribe(name, func)
+
+        return f
+
+    return wrapper
+
+
+class Hook:
+    def __init__(self, name: str, doc: str = "", func: Callable | None = None) -> None:
+        self.name = name
+        self.doc = doc
+        self.func = func
+
+
+class HookHandlerCollection:
+    def __init__(self, registry_name: str, check_name=True):
+        self.hooks: dict[str, HookHandler] = {}
+        if check_name and registry_name in subscriptions:
+            raise NameError("A hook registry already exists with that name: {registry_name}")
+        elif registry_name not in subscriptions:
+            subscriptions[registry_name] = {}
+        self.registry_name = registry_name
+
+    def __getattr__(self, name: str) -> HookHandler:
+        if name not in self.hooks:
+            raise AttributeError
+        return self.hooks[name]
+
+    def _register(self, hook: Hook) -> None:
+        def _hook_func(func):
+            return self._subscribe(hook.name, func)
+
+        hooked = _hook_func if hook.func is None else hook.func(self)
+        hooked.__doc__ = hook.doc
+
+        self.hooks[hook.name] = hooked
+
+
+class Subscribe(HookHandlerCollection):
+    def _subscribe(self, event: str, func: Callable) -> Callable:
+        registry = subscriptions.setdefault(self.registry_name, dict())
+        lst = registry.setdefault(event, [])
         if func not in lst:
             lst.append(func)
         return func
 
-    def startup_once(self, func):
+
+class Unsubscribe(HookHandlerCollection):
+    """
+    This class mirrors subscribe, except the _subscribe member has been
+    overridden to remove calls from hooks.
+    """
+
+    def _subscribe(self, event: str, func: Callable) -> None:
+        registry = subscriptions.setdefault(self.registry_name, dict())
+        lst = registry.setdefault(event, [])
+        try:
+            lst.remove(func)
+        except ValueError:
+            raise utils.QtileError(
+                "Tried to unsubscribe a hook that was not currently subscribed"
+            )
+
+
+class Registry:
+    def __init__(self, name: str, hooks: list[Hook] = list()) -> None:
+        self.name = name
+        self.subscribe = Subscribe(name)
+        self.unsubscribe = Unsubscribe(name, check_name=False)
+        for hook in hooks:
+            self.register_hook(hook)
+
+    def register_hook(self, hook: Hook) -> None:
+        if hook.name in self.subscribe.hooks:
+            raise utils.QtileError(
+                f"Unable to register hook. A hook with that name already exists: {hook.name}"
+            )
+
+        logger.debug("Registered new hook: '%s'.", hook.name)
+        self.subscribe._register(hook)
+        self.unsubscribe._register(hook)
+
+    def fire(self, event, *args, **kwargs):
+        if event not in self.subscribe.hooks:
+            raise utils.QtileError(f"Unknown event: {event}")
+        # Do not fire for Internal windows
+        if any(isinstance(arg, backend.base.window.Internal) for arg in args):
+            return
+        # We should check if the registry name is in the subscriptions dict
+        # A name can disappear if the config is reloaded (which clears subscriptions)
+        # but there are no hook subscriptions. This is not an issue for qtile core but
+        # third party libraries will need this to prevent KeyErrors when firing hooks
+        if self.name not in subscriptions:
+            subscriptions[self.name] = dict()
+        for i in subscriptions[self.name].get(event, []):
+            try:
+                if asyncio.iscoroutinefunction(i):
+                    _fire_async_event(i(*args, **kwargs))
+                elif asyncio.iscoroutine(i):
+                    _fire_async_event(i)
+                else:
+                    i(*args, **kwargs)
+            except:  # noqa: E722
+                logger.exception("Error in hook %s", event)
+
+
+hooks: list[Hook] = [
+    Hook(
+        "startup_once",
         """Called when Qtile has started on first start
 
         This hook is called exactly once per session (i.e. not on each
@@ -80,14 +219,14 @@ class Subscribe:
 
 
           @hook.subscribe.startup_once
-          def autostart:
+          def autostart():
               script = os.path.expanduser("~/.config/qtile/autostart.sh")
               subprocess.run([script])
 
-        """
-        return self._subscribe("startup_once", func)
-
-    def startup(self, func):
+        """,
+    ),
+    Hook(
+        "startup",
         """
         Called when qtile is started. Unlike ``startup_once``, this hook is
         fired on every start, including restarts.
@@ -114,10 +253,10 @@ class Subscribe:
           def run_every_startup():
               send_notification("qtile", "Startup")
 
-        """
-        return self._subscribe("startup", func)
-
-    def startup_complete(self, func):
+        """,
+    ),
+    Hook(
+        "startup_complete",
         """
         Called when qtile is started after all resources initialized.
 
@@ -142,10 +281,10 @@ class Subscribe:
           def run_every_startup():
               send_notification("qtile", "Startup complete")
 
-        """
-        return self._subscribe("startup_complete", func)
-
-    def shutdown(self, func):
+        """,
+    ),
+    Hook(
+        "shutdown",
         """
         Called before qtile is shutdown.
 
@@ -175,10 +314,10 @@ class Subscribe:
               script = os.path.expanduser("~/.config/qtile/shutdown.sh")
               subprocess.run([script])
 
-        """
-        return self._subscribe("shutdown", func)
-
-    def restart(self, func):
+        """,
+    ),
+    Hook(
+        "restart",
         """
         Called before qtile is restarted.
 
@@ -201,10 +340,10 @@ class Subscribe:
           def run_every_startup():
               send_notification("qtile", "Restarting...")
 
-        """
-        return self._subscribe("restart", func)
-
-    def setgroup(self, func):
+        """,
+    ),
+    Hook(
+        "setgroup",
         """
         Called when group is put on screen.
 
@@ -229,10 +368,10 @@ class Subscribe:
           def setgroup():
               send_notification("qtile", "Group set")
 
-        """
-        return self._subscribe("setgroup", func)
-
-    def addgroup(self, func):
+        """,
+    ),
+    Hook(
+        "addgroup",
         """
         Called when a new group is added
 
@@ -252,10 +391,10 @@ class Subscribe:
           def group_added(group_name):
               send_notification("qtile", f"New group added: {group_name}")
 
-        """
-        return self._subscribe("addgroup", func)
-
-    def delgroup(self, func):
+        """,
+    ),
+    Hook(
+        "delgroup",
         """
         Called when group is deleted
 
@@ -275,10 +414,10 @@ class Subscribe:
           def group_deleted(group_name):
               send_notification("qtile", f"Group deleted: {group_name}")
 
-        """
-        return self._subscribe("delgroup", func)
-
-    def changegroup(self, func):
+        """,
+    ),
+    Hook(
+        "changegroup",
         """
         Called whenever a group change occurs.
 
@@ -304,10 +443,10 @@ class Subscribe:
           def change_group():
               send_notification("qtile", "Change group event")
 
-        """
-        return self._subscribe("changegroup", func)
-
-    def focus_change(self, func):
+        """,
+    ),
+    Hook(
+        "focus_change",
         """
         Called when focus is changed, including moving focus between groups or when
         focus is lost completely (i.e. when a window is closed.)
@@ -328,10 +467,10 @@ class Subscribe:
           def focus_changed():
               send_notification("qtile", "Focus changed.")
 
-        """
-        return self._subscribe("focus_change", func)
-
-    def float_change(self, func):
+        """,
+    ),
+    Hook(
+        "float_change",
         """
         Called when a change in float state is made (e.g. toggle floating,
         minimised and fullscreen states)
@@ -352,10 +491,10 @@ class Subscribe:
           def float_change():
               send_notification("qtile", "Window float state changed.")
 
-        """
-        return self._subscribe("float_change", func)
-
-    def group_window_add(self, func):
+        """,
+    ),
+    Hook(
+        "group_window_add",
         """Called when a new window is added to a group
 
         **Arguments**
@@ -375,10 +514,33 @@ class Subscribe:
           def group_window_add(group, window):
               send_notification("qtile", f"Window {window.name} added to {group.name}")
 
-        """
-        return self._subscribe("group_window_add", func)
+        """,
+    ),
+    Hook(
+        "group_window_remove",
+        """Called when a window is removed from a group
 
-    def client_new(self, func):
+        **Arguments**
+
+            * ``Group`` removing the window
+            * ``Window`` removed from the group
+
+        Example:
+
+        .. code:: python
+
+          from libqtile import hook
+          from libqtile.utils import send_notification
+
+
+          @hook.subscribe.group_window_remove
+          def group_window_remove(group, window):
+              send_notification("qtile", f"Window {window.name} removed from {group.name}")
+
+        """,
+    ),
+    Hook(
+        "client_new",
         """
         Called before Qtile starts managing a new client
 
@@ -403,10 +565,10 @@ class Subscribe:
                 elif client.name == "dzen":
                     client.static(0)
 
-        """
-        return self._subscribe("client_new", func)
-
-    def client_managed(self, func):
+        """,
+    ),
+    Hook(
+        "client_managed",
         """
         Called after Qtile starts managing a new client
 
@@ -428,12 +590,13 @@ class Subscribe:
             def client_managed(client):
                 send_notification("qtile", f"{client.name} has been managed by qtile")
 
+        """,
+    ),
+    Hook(
+        "client_killed",
         """
-        return self._subscribe("client_managed", func)
-
-    def client_killed(self, func):
-        """
-        Called after a client has been unmanaged
+        Called after a client has been unmanaged. This hook is not called for
+        internal windows.
 
         **Arguments**
 
@@ -450,10 +613,10 @@ class Subscribe:
             def client_killed(client):
                 send_notification("qtile", f"{client.name} has been killed")
 
-        """
-        return self._subscribe("client_killed", func)
-
-    def client_focus(self, func):
+        """,
+    ),
+    Hook(
+        "client_focus",
         """
         Called whenever focus moves to a client window
 
@@ -472,10 +635,10 @@ class Subscribe:
             def client_focus(client):
                 send_notification("qtile", f"{client.name} has been focused")
 
-        """
-        return self._subscribe("client_focus", func)
-
-    def client_mouse_enter(self, func):
+        """,
+    ),
+    Hook(
+        "client_mouse_enter",
         """
         Called when the mouse enters a client
 
@@ -490,14 +653,14 @@ class Subscribe:
             from libqtile import hook
             from libqtile.utils import send_notification
 
-            @hook.subscribe.client_killed
+            @hook.subscribe.client_mouse_enter
             def client_mouse_enter(client):
                 send_notification("qtile", f"Mouse has entered {client.name}")
 
-        """
-        return self._subscribe("client_mouse_enter", func)
-
-    def client_name_updated(self, func):
+        """,
+    ),
+    Hook(
+        "client_name_updated",
         """
         Called when the client name changes
 
@@ -519,10 +682,10 @@ class Subscribe:
                     f"Client's has been updated to {client.name}"
                 )
 
-        """
-        return self._subscribe("client_name_updated", func)
-
-    def client_urgent_hint_changed(self, func):
+        """,
+    ),
+    Hook(
+        "client_urgent_hint_changed",
         """
         Called when the client urgent hint changes
 
@@ -544,10 +707,10 @@ class Subscribe:
                     f"{client.name} has changed its urgency state"
                 )
 
-        """
-        return self._subscribe("client_urgent_hint_changed", func)
-
-    def layout_change(self, func):
+        """,
+    ),
+    Hook(
+        "layout_change",
         """
         Called on layout change event (including when a new group is
         displayed on the screen)
@@ -570,10 +733,10 @@ class Subscribe:
                     "qtile",
                     f"{layout.name} is now on group {group.name}"
                 )
-        """
-        return self._subscribe("layout_change", func)
-
-    def net_wm_icon_change(self, func):
+        """,
+    ),
+    Hook(
+        "net_wm_icon_change",
         """
         Called on ``_NET_WM_ICON`` change
 
@@ -595,10 +758,10 @@ class Subscribe:
             def icon_change(client):
                 send_notification("qtile", f"{client.name} has changed its icon")
 
-        """
-        return self._subscribe("net_wm_icon_change", func)
-
-    def selection_notify(self, func):
+        """,
+    ),
+    Hook(
+        "selection_notify",
         """
         Called on selection notify
 
@@ -627,10 +790,10 @@ class Subscribe:
                     f"Window {selection['owner']} has made a selection in the {name} selection."
                 )
 
-        """
-        return self._subscribe("selection_notify", func)
-
-    def selection_change(self, func):
+        """,
+    ),
+    Hook(
+        "selection_change",
         """
         Called on selection change
 
@@ -660,10 +823,10 @@ class Subscribe:
                     f"Window {selection['owner']} has changed the {name} selection."
                 )
 
-        """
-        return self._subscribe("selection_change", func)
-
-    def screen_change(self, func):
+        """,
+    ),
+    Hook(
+        "screen_change",
         """
         Called when the output configuration is changed (e.g. via randr in X11).
 
@@ -690,10 +853,10 @@ class Subscribe:
             def screen_change(event):
                 send_notification("qtile", "Screen change detected.")
 
-        """
-        return self._subscribe("screen_change", func)
-
-    def screens_reconfigured(self, func):
+        """,
+    ),
+    Hook(
+        "screens_reconfigured",
         """
         Called once ``qtile.reconfigure_screens`` has completed (e.g. if
         ``reconfigure_screens`` is set to ``True`` in your config).
@@ -709,14 +872,14 @@ class Subscribe:
             from libqtile import hook
             from libqtile.utils import send_notification
 
-            @hook.subscribe.screen_change
-            def screen_change(event):
+            @hook.subscribe.screens_reconfigured
+            def screen_reconf():
                 send_notification("qtile", "Screens have been reconfigured.")
 
-        """
-        return self._subscribe("screens_reconfigured", func)
-
-    def current_screen_change(self, func):
+        """,
+    ),
+    Hook(
+        "current_screen_change",
         """
         Called when the current screen (i.e. the screen with focus) changes
 
@@ -732,13 +895,13 @@ class Subscribe:
             from libqtile.utils import send_notification
 
             @hook.subscribe.current_screen_change
-            def screen_change(event):
+            def screen_change():
                 send_notification("qtile", "Current screen change detected.")
 
-        """
-        return self._subscribe("current_screen_change", func)
-
-    def enter_chord(self, func):
+        """,
+    ),
+    Hook(
+        "enter_chord",
         """
         Called when key chord begins
 
@@ -760,10 +923,10 @@ class Subscribe:
             def enter_chord(chord_name):
                 send_notification("qtile", "Started {chord_name} key chord.")
 
-        """
-        return self._subscribe("enter_chord", func)
-
-    def leave_chord(self, func):
+        """,
+    ),
+    Hook(
+        "leave_chord",
         """
         Called when key chord ends
 
@@ -782,10 +945,10 @@ class Subscribe:
             ded leave_chord():
                 send_notification("qtile", "Key chord exited")
 
-        """
-        return self._subscribe("leave_chord", func)
-
-    def resume(self, func):
+        """,
+    ),
+    Hook(
+        "resume",
         """
         Called when system wakes up from sleep, suspend or hibernate.
 
@@ -797,11 +960,11 @@ class Subscribe:
         **Arguments**
 
         None
-        """
-        inhibitor.want_resume()
-        return self._subscribe("resume", func)
-
-    def suspend(self, func):
+        """,
+        _resume_func,
+    ),
+    Hook(
+        "suspend",
         """
         Called when system is about to sleep, suspend or hibernate.
 
@@ -846,58 +1009,53 @@ class Subscribe:
               # Run screen locker
               qtile.spawn("/path/to/screen_locker")
 
+        """,
+        _suspend_func,
+    ),
+    Hook(
+        "user",
         """
-        inhibitor.want_sleep()
-        return self._subscribe("suspend", func)
+        Use to create user-defined hooks.
+
+        The purpose of these hooks is to allow a hook to be fired by an external application.
+
+        Hooked functions can receive arguments but it is up to the application firing the hook to ensure
+        the correct arguments are passed. No checking will be performed by qtile.
+
+        Example:
+
+        .. code:: python
+
+          from libqtile import hook
+          from libqtile.log_utils import logger
+
+          @hook.subscribe.user("my_custom_hook")
+          def hooked_function():
+            logger.warning("Custom hook received.")
+
+        The external script can then call the hook with the following command:
+
+        .. code::
+
+          qtile cmd-obj -o cmd -f fire_user_hook -a my_custom_hook
+
+        .. note::
+
+          If the script will be run by a different user then you will need to pass the path to the socket
+          file used by the current process. One way to achieve this is to specify a path for the socket when starting
+          qtile e.g. ``qtile start -s /tmp/qtile.socket``.
+          When firing the hook, you should then call
+          ``qtile cmd-obj -o cmd -f fire_user_hook -a my_custom_hook -s /tmp/qtile.socket``
+          However, the same socket will need to be passed wherever you run ``qtile cmd-obj`` or ``qtile shell``.
+
+        """,
+        _user_hook_func,
+    ),
+]
 
 
-subscribe = Subscribe()
+qtile_hooks = Registry("qtile", hooks)
 
-
-class Unsubscribe(Subscribe):
-    """
-    This class mirrors subscribe, except the _subscribe member has been
-    overridden to removed calls from hooks.
-    """
-
-    def _subscribe(self, event, func):
-        lst = subscriptions.setdefault(event, [])
-        try:
-            lst.remove(func)
-        except ValueError:
-            raise utils.QtileError(
-                "Tried to unsubscribe a hook that was not currently subscribed"
-            )
-
-
-unsubscribe = Unsubscribe()
-
-
-def _fire_async_event(co):
-    from libqtile.utils import create_task
-
-    loop = None
-    with contextlib.suppress(RuntimeError):
-        loop = asyncio.get_running_loop()
-
-    if loop is None:
-        asyncio.run(co)
-    else:
-        create_task(co)
-
-
-def fire(event, *args, **kwargs):
-    if event not in subscribe.hooks:
-        raise utils.QtileError("Unknown event: %s" % event)
-    if event not in SKIPLOG:
-        logger.debug("Internal event: %s(%s, %s)", event, args, kwargs)
-    for i in subscriptions.get(event, []):
-        try:
-            if asyncio.iscoroutinefunction(i):
-                _fire_async_event(i(*args, **kwargs))
-            elif asyncio.iscoroutine(i):
-                _fire_async_event(i)
-            else:
-                i(*args, **kwargs)
-        except:  # noqa: E722
-            logger.exception("Error in hook %s", event)
+subscribe = qtile_hooks.subscribe
+unsubscribe = qtile_hooks.unsubscribe
+fire = qtile_hooks.fire
