@@ -22,12 +22,11 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-import asyncio
 from typing import Any
 
 try:
-    from dbus_next import ReleaseNameReply
     from dbus_next.aio import MessageBus
+    from dbus_next.constants import NameFlag, RequestNameReply
     from dbus_next.service import ServiceInterface, method, signal
 
     has_dbus = True
@@ -63,7 +62,7 @@ if has_dbus:
         def register_capabilities(self, capabilities):
             if isinstance(capabilities, str):
                 self._capabilities.add(capabilities)
-            elif isinstance(capabilities, (tuple, list, set)):
+            elif isinstance(capabilities, tuple | list | set):
                 self._capabilities.update(set(capabilities))
 
         @method()
@@ -74,8 +73,8 @@ if has_dbus:
             app_icon: "s",  # type:ignore  # noqa: F821
             summary: "s",  # type:ignore  # noqa: F821
             body: "s",  # type:ignore  # noqa: F821
-            actions: "as",  # type:ignore  # noqa: F821
-            hints: "a{sv}",  # type:ignore  # noqa: F821
+            actions: "as",  # type:ignore  # noqa: F722, F821
+            hints: "a{sv}",  # type:ignore  # noqa: F722, F821
             timeout: "i",  # type:ignore  # noqa: F821
         ) -> "u":  # type:ignore  # noqa: F821
             notif = Notification(
@@ -84,18 +83,22 @@ if has_dbus:
             return self.manager.add(notif)
 
         @method()
-        def CloseNotification(self, nid: "u"):  # type:ignore  # noqa: N802, F821
+        def CloseNotification(self, nid: "u"):  # type:ignore  # noqa: F821, N802
             self.manager.close(nid)
 
         @signal()
         def NotificationClosed(  # noqa: N802
-            self, nid: "u", reason: "u"  # type: ignore  # noqa: F821
+            self,
+            nid: "u",  # type: ignore  # noqa: F821
+            reason: "u",  # type: ignore  # noqa: F821
         ) -> "uu":  # type: ignore  # noqa: F821
             return [nid, reason]
 
         @signal()
         def ActionInvoked(  # noqa: N802
-            self, nid: "u", action_key: "s"  # type: ignore  # noqa: F821
+            self,
+            nid: "u",  # type: ignore  # noqa: F821
+            action_key: "s",  # type: ignore  # noqa: F821
         ) -> "us":  # type:ignore  # noqa: N802, F821
             return [nid, action_key]
 
@@ -130,17 +133,42 @@ if has_dbus:
             self.callbacks = []
             self.close_callbacks = []
             self._service = None
+            self.bus = None
 
         async def service(self):
-            if self._service is None:
-                try:
-                    self.bus = await MessageBus().connect()
+            if not self.callbacks:
+                if self.bus is None:
+                    try:
+                        self.bus = await MessageBus().connect()
+                    except Exception:
+                        logger.exception("Dbus connection failed")
+                        self._service = None
+                        return self._service
+
                     self._service = NotificationService(self)
                     self.bus.export(SERVICE_PATH, self._service)
-                    await self.bus.request_name(BUS_NAME)
-                except Exception:
-                    logger.exception("Dbus connection failed")
+
+                reply = await self.bus.request_name(
+                    BUS_NAME,
+                    flags=NameFlag.ALLOW_REPLACEMENT
+                    | NameFlag.REPLACE_EXISTING
+                    | NameFlag.DO_NOT_QUEUE,
+                )
+
+                # Check the reply to see if another server is already running.
+                # Other replies could be RequestNameReply.PRIMARY_OWNER or
+                # RequestNameReply.ALREADY_ONWER. Both would indicate qtile server
+                # is running successfully so we don't need to check for them.
+                # RequestNameReply.IN_QUEUE should not be possible as we pass
+                # NameFlag.DO_NOT_QUEUE when requesting the name.
+                if reply == RequestNameReply.EXISTS:
+                    logger.warning(
+                        "Cannot start notification server as another server is already running."
+                    )
                     self._service = None
+                    self.bus.disconnect()
+                    self.bus = None
+
             return self._service
 
         async def register(self, callback, capabilities=None, on_close=None):
@@ -168,23 +196,6 @@ if has_dbus:
                 except ValueError:
                     logger.error("Unable to remove notify on_close callback. Unknown callback.")
 
-            if not self.callbacks:
-                return asyncio.create_task(self._release())
-
-        async def _release(self):
-            """
-            If the manager has no more callbacks then we need to release the service name
-            from dbus and reset _service to None (to force subsequent calls to `register` to
-            re-register the name on dbus.)
-            """
-            reply = await self.bus.release_name(BUS_NAME)
-
-            if reply != ReleaseNameReply.RELEASED:
-                logger.error("Could not release %s", BUS_NAME)
-                return
-
-            self._service = None
-
         def add(self, notif):
             self.notifications.append(notif)
             notif.id = len(self.notifications)
@@ -200,11 +211,9 @@ if has_dbus:
             return (notif, self.add(notif))
 
         def close(self, nid):
-            notif = self.notifications[nid]
-
             for callback in self.close_callbacks:
                 try:
-                    callback(notif)
+                    callback(nid)
                 except Exception:
                     logger.exception("Exception in notifier close callback")
 
